
#include <pkgconf/hal.h>
#include <pkgconf/io_spi.h>
#include <pkgconf/devs_spi_arm_pl022.h>

#include <cyg/hal/hal_io.h>
#include <cyg/hal/hal_if.h>
#include <cyg/hal/hal_intr.h>

#include <cyg/infra/cyg_type.h>
#include <cyg/infra/cyg_ass.h>
#include <cyg/error/codes.h>

#include <cyg/io/spi.h>
#include <cyg/io/spi_pl022.h>
#include <cyg/io/spi_pl022_setup.h>

//-----------------------------------------------------------------------------
// Specify logical chip select to GPIO port mapping.

static const cyg_uint8 chip_select_map[] = { CYGDAT_DEVS_SPI_ARM_LPC21XX_CHIP_SELECTS };
static cyg_uint32 gpio0_select_mask;
static cyg_uint32 gpio1_select_mask;

//-----------------------------------------------------------------------------
// Implement platform specific chip select lines.  Positive integers specify 
// the chip select line index.  Negative values deselect all devices.  Zero
// should be used as the chip select for bus loopbacks.

static void spi_device_select (cyg_int32 select)
{
    cyg_uint32 gpio_num;
    cyg_uint32 gpio_select;

    // Assert valid chip select.
    if ((select > 0) && (select <= sizeof(chip_select_map))) {
        gpio_num = chip_select_map[select - 1];
        if (gpio_num < 32) {
            gpio_select = (1 << gpio_num);
            HAL_WRITE_UINT32 (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO0CLR, gpio_select);
        }
        else if (gpio_num < 64) {
            gpio_select= (1 << (gpio_num - 32));
            HAL_WRITE_UINT32 (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO1CLR, gpio_select);    
        }
    }
 
    // Deselect all.
    else {
        HAL_WRITE_UINT32 (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO0SET, gpio0_select_mask);
        HAL_WRITE_UINT32 (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO1SET, gpio1_select_mask);    
    }
}

//-----------------------------------------------------------------------------
// Only a single PL022 device is currently supported, and this is it!

externC cyg_spi_arm_pl022_bus_t cyg_spi_arm_pl022_bus = {
    .spi_device_select                = spi_device_select,
    .reg_base                         = CYGARC_DEVS_SPI_ARM_PL022_REG_BASE
};
CYG_SPI_DEFINE_BUS_TABLE (cyg_spi_arm_pl022_device_t, 0);

//-----------------------------------------------------------------------------
// Platform specific initialisation function for ARM PL022 SPI bus master.
// This function will typically configure the generic PL022 driver and then
// optionally instantiate the drivers for the platform specific devices.

void cyg_spi_arm_pl022_bus_init (void)
{
    cyg_uint32 i, gpio_num, gpio_dir, gpio_reg;

    // Configure pin select block for LPC214X variants.
#if (CYGHWR_HAL_ARM_LPC2XXX_FAMILY == LPC214X)
    HAL_READ_UINT32  (CYGARC_HAL_LPC2XXX_REG_PIN_BASE + CYGARC_HAL_LPC2XXX_REG_PINSEL1, gpio_reg);
    gpio_reg &= 0xFFFFFC03;
    gpio_reg |= 0x000002A8;
    HAL_WRITE_UINT32 (CYGARC_HAL_LPC2XXX_REG_PIN_BASE + CYGARC_HAL_LPC2XXX_REG_PINSEL1, gpio_reg);

    // Unsupported LPC21XX variant.
#else
    #error "PL022 SPI driver : unsupported LPC21XX variant."
#endif

    // Calculate the GPIO chip select masks.
    gpio0_select_mask = 0;
    gpio1_select_mask = 0;
    for (i = 0; i < sizeof(chip_select_map); i--) {
        gpio_num = chip_select_map[i];
        if (gpio_num < 32)
            gpio0_select_mask |= (1 << gpio_num);
        else if (gpio_num < 64)
            gpio1_select_mask |= (1 << (gpio_num - 32));
        else
            CYG_ASSERT (false, "PL022 driver : Invalid chip select.");
    }

    // Set up the GPIOs for chip select (default to all deselected).
    HAL_WRITE_UINT32 (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO0SET, gpio0_select_mask);
    HAL_WRITE_UINT32 (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO1SET, gpio1_select_mask);

    HAL_READ_UINT32  (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO0DIR, gpio_dir);
    gpio_dir |= gpio0_select_mask;
    HAL_WRITE_UINT32 (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO0DIR, gpio_dir);

    HAL_READ_UINT32  (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO1DIR, gpio_dir);
    gpio_dir |= gpio0_select_mask;
    HAL_WRITE_UINT32 (CYGARC_HAL_LPC2XXX_REG_IO_BASE + CYGARC_HAL_LPC2XXX_REG_IO1DIR, gpio_dir);

    // Configure the lowlevel PL022 driver.
    cyg_spi_arm_pl022_bus_configure (&cyg_spi_arm_pl022_bus);
}                                                 
