#ifndef CYGONCE_DEVS_SPI_ARM_PL022_H
#define CYGONCE_DEVS_SPI_ARM_PL022_H

//-----------------------------------------------------------------------------

#include <pkgconf/hal.h>
#include <pkgconf/io_spi.h>

#include <cyg/infra/cyg_type.h>
#include <cyg/hal/drv_api.h>
#include <cyg/io/spi.h>

//-----------------------------------------------------------------------------
// Function prototype for device select callbacks to the platform layer.
// Passes an integer parameter which selects the active device, or is set
// to -1 to deselect all devices.

typedef void (*cyg_spi_device_select_callback_t) (cyg_int32);

//-----------------------------------------------------------------------------
// ARM Primecell PL022 SPI bus state

typedef struct cyg_spi_arm_pl022_bus_s
{
    // ---- Upper layer data ----
    cyg_spi_bus   spi_bus;             // Upper layer SPI bus data

    // ---- Platform setup ----
    cyg_spi_device_select_callback_t spi_device_select;
                                       // Device select callback to platform layer
    cyg_haladdress    reg_base;        // Base address of PL022 register block

    // ---- Lower layer data ----
    cyg_interrupt     intr_data;       // SPI interrupt data
    cyg_handle_t      intr_handle;     // SPI interrupt handle
    cyg_drv_mutex_t   tfer_mutex;      // Transfer mutex
    cyg_drv_cond_t    tfer_condvar;    // Transfer condition variable
    cyg_uint32        tx_count;        // Count of remaining bytes to transmit
    cyg_uint32        rx_count;        // Count of remaining bytes to receive
    const cyg_uint8*  tx_data_ptr;     // Pointer to current transmit data
    cyg_uint8*        rx_data_ptr;     // Pointer to current receive data
    cyg_uint8         bus_16bit;       // Use 16 bit (1) or 8 bit (0) transfers
    cyg_bool          cs_up;           // Chip select up flag 

} cyg_spi_arm_pl022_bus_t;

//-----------------------------------------------------------------------------
// ARM Primecell PL022 SPI device

typedef struct cyg_spi_arm_pl022_device_s
{
    // ---- Upper layer data ----
    cyg_spi_device spi_device;  // Upper layer SPI device data

    // ---- Lower layer data (configurable) ----
    cyg_uint8  dev_num;         // Device number
    cyg_uint8  cl_pol;          // Clock polarity (0 or 1)
    cyg_uint8  cl_pha;          // Clock phase    (0 or 1)
    cyg_uint32 cl_brate;        // Clock baud rate
    cyg_uint16 cs_up_udly;      // Minimum delay in us between CS up and transfer start
    cyg_uint16 cs_dw_udly;      // Minimum delay in us between transfer end and CS down
    cyg_uint16 tr_bt_udly;      // Minimum delay in us between two transfers

    cyg_uint8  bus_loopback;    // Select SPI bus loopback for testing
    cyg_uint8  bus_16bit;       // Use 16 bit (1) or 8 bit (0) transfers

    // ---- Lower layer data (internal) ----
    cyg_bool   init;            // Is device initialized
    cyg_uint8  sspcr0_scr;      // Pre-calculated value for clock divider 

} cyg_spi_arm_pl022_device_t;

//-----------------------------------------------------------------------------
// Configuration function for new PL022 bus.  This is called from the platform
// initialisation, which must also set up the associated GPIO pin mappings.

externC void cyg_spi_arm_pl022_bus_configure (
    cyg_spi_arm_pl022_bus_t* pl022_bus   // Pointer to new PL022 bus data.
);                                                 
    
//-----------------------------------------------------------------------------

#endif // CYGONCE_DEVS_SPI_ARM_PL022_H 

//-----------------------------------------------------------------------------
// End of spi_pl022.h
